//----------------------------------------------------------------------------
//  EDGE Main
//----------------------------------------------------------------------------
// 
//  Copyright (c) 1999-2006  The EDGE Team.
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------
#include "edge/i_sdlinc.h"  // needed for proper SDL main linkage

#include "mem.h"
#include "profiler.h"

//#define RUN_GOLDENEYE

using namespace std;

void *operator new(size_t sz)
{
	return mem_malloc(sz,"operator new sin registrar");
}

void operator delete(void *m)
{
	mem_free(m,"operator delete sin registrar");
}

#ifdef PROFILER_DREAMEDGE
static unsigned prof_total_initial=0;
unsigned prof_total=0;
static char *prof_msg[DREAMEDGE_PROFILER_MAX];
unsigned long long prof_initial[DREAMEDGE_PROFILER_MAX];
unsigned long long prof_sum[DREAMEDGE_PROFILER_MAX];
unsigned long long prof_executed[DREAMEDGE_PROFILER_MAX];
int prof_started[DREAMEDGE_PROFILER_MAX];

void prof_reset(void)
{
	int i;
	unsigned long long s=get_time_in_micros();
	for(i=0;i<DREAMEDGE_PROFILER_MAX;i++)
	{
		prof_initial[i]=s;
		prof_sum[i]=0;
		if (prof_started[i])
			prof_executed[i]=1;
		else
			prof_executed[i]=0;
	}
	prof_total_initial=s;
}

void prof_init(void)
{
	unsigned i;
	unsigned long long s=get_time_in_micros();
	for(i=0;i<DREAMEDGE_PROFILER_MAX;i++)
	{
		prof_initial[i]=s;
		prof_sum[i]=0;
		if (!prof_total)
		{
			prof_executed[i]=0;
			prof_started[i]=0;
			prof_msg[i]=NULL;
		}
		else
		{
			if (prof_started[i])
			        prof_executed[i]=1;
			else
			        prof_executed[i]=0;
		}
	}
	prof_total_initial=s;
}

void prof_add(char *msg)
{
	if (prof_total<DREAMEDGE_PROFILER_MAX)
	{
		prof_msg[prof_total]=msg;
		prof_total++;
	}
}

void prof_show(void)
{
	int i;
	double toper=0;
	unsigned long long to;
	for(i=prof_total-1;i>=0;i--)
		prof_end(i);
	to=prof_sum[0];
	puts("\n\n\n\n");
	puts("--------------------------------------------");
	for(i=0;i<prof_total;i++)
	{
		unsigned long long t0=prof_sum[i];
		double percent=(double)t0;
		percent*=100.0;
		percent/=(double)to;
		toper+=percent;
#ifndef PROFILER_SDL
		t0/=1000;
#endif
		printf("%s: %.2f%% -> Ticks=%i ->",prof_msg[i],percent,((unsigned)t0));
		if (prof_executed[i]>10000LL)
			printf("%iK veces\n",(unsigned)(prof_executed[i]/1000LL));
		else
			printf("%i veces\n",(unsigned)(prof_executed[i]));
	}
//      printf("TOTAL: %.2f%% -> Ticks=%i\n",toper,to);
	puts("--------------------------------------------"); fflush(stdout);
}

void prof_setmsg(int n, char *msg)
{
	prof_msg[n]=msg;
}

#endif



extern "C" {

extern int I_Main(int argc, char *argv[]);

int main(int argc, char *argv[])
{
	cache_init();
#ifdef PROFILER_DREAMEDGE
	prof_init();
	prof_add("Main");	// PROF_TOTAL
	prof_add("Blit");	// PROF_BLIT
	prof_add("Render");	// PROF_RENDER
	prof_add("DrawSeg");	// PROF_DRAWSEG
	prof_add("DrawUnits");	// PROF_DRAWUNITS
	prof_add("R0");		// PROF_R0
	prof_add("R1");		// PROF_R1
	prof_add("R2");		// PROF_R2
	prof_add("R3");		// PROF_R3
	prof_add("R4");		// PROF_R4
	prof_add("R5");		// PROF_R5
	prof_add("R6");		// PROF_R6
	prof_add("R7");		// PROF_R7
#endif
#ifdef DREAMCAST
	char *a[]={"dreamedge","-width","640","-height","480","-bpp","16","-fullscreen","-smoothing","-nomipmap","-nomusic",
#ifdef NO_SOUND
		"-nosound",
#endif
#ifdef RUN_GOLDENEYE
	   	//"-file","gold64.wad","ge64levs.wad",
	   	"-file","ge64ddf.wad","ge64levs.wad",
#endif
		"-nomusic","-nomusic","-nomusic","-nomusic","-nomusic","-nomusic","-nomusic",NULL};
	puts("MAIN!!!");fflush(stdout);
	return I_Main(16,a);
#else
	return I_Main(argc, argv);
#endif
}

} // extern "C"


//--- editor settings ---
// vi:ts=4:sw=4:noexpandtab


