
#ifndef __MEM_H__
#define __MEM_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef MEM_CACHE
void *cache_calloc(size_t nmemb, size_t size);
void *cache_malloc(size_t real_size);
void cache_free(void *ptr);
void *cache_realloc(void *ptr, size_t size);
char *cache_strdup(const char *s);
char *cache_strndup(const char *s, size_t size);
void cache_init(void);
#else
#define cache_calloc(N,S) calloc(N,S)
#define cache_malloc(S) malloc(S)
#define cache_free(P) free(P)
#define cache_realloc(P,S) realloc(P,S)
#define cache_strdup(S) strdup(S)
#define cache_strndup(S,Z) strndup(S,Z)
#define cache_init()
#endif

#ifdef MEM_STATS
void *mem_calloc(size_t nmemb, size_t size, const char *msg);
void *mem_malloc(size_t size, const char *msg);
void mem_free(void *ptr, const char *msg);
void *mem_realloc(void *ptr, size_t size, const char *msg);
char *mem_strdup(const char *s, const char *msg);
char *mem_strndup(const char *s, size_t size, const char *msg);
void print_meminfo(const char *msg);
#else
#define mem_calloc(N,S,MSG) cache_calloc(N,S)
#define mem_malloc(S,MSG) cache_malloc(S)
#define mem_free(P,MSG) cache_free(P)
#define mem_realloc(P,S,MSG) cache_realloc(P,S)
#define mem_strdup(S,MSG) cache_strdup(S)
#define mem_strndup(S,Z,MSG) cache_strndup(S,Z)
#define print_meminfo(MSG)
#endif

#ifdef __cplusplus
}
#endif

#endif

